/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;

/**
 * A class that produces a Bevel.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class Bevel extends Canvas {
	/** Horizontal bevel. This constant is use for setting
	 *  direction variable.
	 * @see #DIR_VERTICAL
	 * @see #setDirection
	 * @see #getDirection
	 */
	public final static boolean DIR_HORIZONTAL = true;

	/** Vertical bevel. This constant is use for setting
	 *  direction variable.
	 * @see #DIR_HORIZONTAL
	 * @see #setDirection
	 * @see #getDirection
	 */
	public final static boolean DIR_VERTICAL = false;

	/** Current direction of Bevel. It can be either DIR_HORIZONTAL
	 *  or DIR_VERTICAL. For setting the value use setDirection
	 *  method
	 * @see #DIR_HORIZONTAL
	 * @see #DIR_VERTICAL
	 * @see #defaultDirection
	 * @see #setDirection
	 * @see #getDirection
	 */
    protected boolean direction;

	/** Default value of direction variable. When Bevel is created
	 *  direction is set to this value. 
	 * @see #DIR_HORIZONTAL
	 * @see #direction
	 */
	public static final boolean defaultDirection = DIR_HORIZONTAL;

	/** The basic color of the 3D border. Left and top sides are 
	 *  lighter and right and bottom sides are darker then this 
	 *  color. By default it is set to defaultBorderColor.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #defaultBorderColor
	 */
	protected Color marginColor = null;

	/** Default color of the border. Currently lightGray.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #marginColor
	 */
	public static final Color defaultBorderColor = new Color(192, 192, 192);

	/** Current width of the 3D border around the component
	 * @see #defaultBorderWidth
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */	
	protected int margin = 0;

	/** Default value of margin. Currently 1.
	 * @see #margin
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */
    public static final int defaultBorderWidth = 1;

	/** 3D raised style of bevel. This constant is use for setting
	 *  style variable.
	 * @see #style
	 */
	public final static boolean ST_RAISED = true;

	/** 3D lowered style of bevel. This constant is use for setting
	 *  style variable.
	 * @see #style
	 */
	public final static boolean ST_LOWERED = false;

	/** Current style of Bevel. It can be either ST_RAISED
	 *  or ST_LOWERED. For setting the value use setStyle method.
	 * @see #ST_RAISED
	 * @see #ST_LOWERED
	 * @see #defaultStyle
	 * @see #setStyle
	 * @see #getStyle
	 */
	protected boolean style;

	/** Default value of style variable. When the Bevel is created
	 *  style is set to this value. 
	 * @see #ST_RAISED
	 * @see #style
	 */
	public boolean defaultStyle = ST_RAISED;

	/** Width of space between the border. Use setBevelWidth for
	 *  setting this variable.
	 * @see #defaultBevelWidth
	 * @see #setBevelWidth
	 * @see #getBevelWidth
	 */
	protected int bevelWidth;

	/** Default value of bevelWidth variable. Currently it is set
	 *  to 0.
	 * @see #bevelWidth
	 * @see #setBevelWidth
	 * @see #getBevelWidth
	 */
	public final static int defaultBevelWidth = 0;

	/** Creates new Bevel with all default values */
	public Bevel() {
		setDefaults();
	}

	private void setDefaults() {
		direction = defaultDirection;
    	marginColor = defaultBorderColor;
		margin = defaultBorderWidth;
		bevelWidth = defaultBevelWidth;
		style = defaultStyle;
	}

	/** Sets the new direction. Allowed values are DIR_xxx constants.
	 * @param aDir new value of direction.
	 * @see #direction
	 * @see #getDirection
	 * @see #DIR_HORIZONTAL
	 * @see #DIR_VERTICAL
	 */
	public void setDirection(boolean aDir) {
		if (aDir == direction)
			return;
		Dimension d = size();
		direction = aDir;
		resize(d.height, d.width);
		checkSize();
		repaint();
	}

	/** Gets the current direction.
	 * @return Direction.
	 * @see #direction
	 * @see #setDirection
	 * @see #DIR_HORIZONTAL
	 * @see #DIR_VERTICAL
	 */
	public boolean getDirection() {
		return direction;
	}

	/** Sets the lenght of the bevel.
	 * @param len New lenght of the bevel.
	 */
	public void setLenght(int len) {
		Dimension d = size();
		if (direction == DIR_HORIZONTAL)
			resize(len, d.height);
		else
			resize(d.width, len);
		checkSize();
		repaint();
	}

	/** Sets the new style. Allowed values are ST_xxx constants.
	 * @param aStyle New value of style.
	 * @see #style
	 * @see #getStyle
	 * @see #ST_RAISED
	 * @see #ST_LOWERED
	 */
	public void setStyle(boolean aStyle) {
		style = aStyle;
		repaint();
	}

	/** Gets the current style.
	 * @return Style.
	 * @see #style
	 * @see #setStyle
	 * @see #ST_RAISED
	 * @see #ST_LOWERED
	 */
	public boolean getStyle() {
		return style;
	}

	/** Sets the border color.
	 * @param col New value of marginColor.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #getBorderColor
	 */
    public void setBorderColor(Color col) {
        marginColor = col;
        repaint();
    }

	/** Gets the border color.
	 * @return Current border color.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #setBorderColor
	 */
    public Color getBorderColor() {
        return marginColor;
    }

	/** Sets border width around the component.
	 * @param borderWidth New value of margin.
	 * @see #margin
	 * @see #getBorderWidth
	 */
    public void setBorderWidth(int borderWidth) {
        margin = borderWidth;
		checkSize();
        repaint();
    }

	/** Gets border width around the component.
	 * @see #margin
	 * @see #setBorderWidth
	 */
    public int getBorderWidth() {
        return(margin);
    }

	/** Sets the new bevel width.
	 * @param aBevelWidth New value of bevelWidth.
	 * @see #bevelWidth
	 * @see #setBevelWidth
	 */
    public void setBevelWidth(int aBevelWidth) {
        bevelWidth = aBevelWidth;
		checkSize();
        repaint();
    }

	/** Gets bevel width.
	 * @return Current value of bevelWidth.
	 * @see #bevelWidth
	 * @see #setBevelWidth
	 */
    public int getBevelWidth() {
        return(bevelWidth);
    }

	private void checkSize() {
		Dimension d = size();
		if (direction == DIR_HORIZONTAL) {
			if (d.width < 2 * margin)
				d.width = 2 * margin;
			int h = bevelWidth + 2 * margin;
			if (h != d.height) {
				d.height = h;
				resize(d);
			}
		}
		else {
			if (d.height < 2 * margin)
				d.height = 2 * margin;
			int w = bevelWidth + 2 * margin;
			if (w != d.width) {
				d.width = w;
				resize(d);
			}
		}
	}

	public void paint(Graphics g) {
		Dimension d = size();

		int x = 0;
		int y = 0;
		int w, h;

		if (direction == DIR_HORIZONTAL) {
			w = d.width - 1;
			h = bevelWidth + 2 * margin - 1;
		}
		else {
			w = bevelWidth + 2 * margin - 1;
			h = d.height - 1;
		}

		g.setColor(marginColor);
		for (int i = 0; i < margin; i++) {
			g.draw3DRect(x, y, w, h, style);
			x++;
			y++;
			w -= 2;
			h -= 2;
		}
	}

    public Dimension minimumSize() {
        return countSize();
    }

    public Dimension preferredSize() {
        return countSize();
    }

	private Dimension countSize() {
		Dimension d = size();
		if (direction == DIR_HORIZONTAL) {
			if (d.width < 2 * margin)
				d.width = 2 * margin;
			d.height = bevelWidth + 2 * margin;
		}
		else {
			if (d.height < 2 * margin)
				d.height = 2 * margin;
			d.width = bevelWidth + 2 * margin;
		}
		return d;
	}

    public void resize(int width, int height) {
		super.resize(width, height);
		checkSize();
		repaint();
    }

    public void resize(Dimension d) {
		super.resize(d);
		checkSize();
		repaint();
    }

    public void reshape(int x, int y, int width, int height) {
		super.reshape(x, y, width, height);
		checkSize();
		repaint();
	}
}
