package xelfi.awt;

import java.awt.*;

/**
 * Implementation of string element which can be viewed in tree
 * shape structure on the screen.
 *
 * @version 1.01, 19 Nov 1996
 * @author Dafe
*/
public class XFStringTreeElement implements XelfiTreeViewElement {

	// horizontal border
	private static final int	XBORDER = 10;
	// string 
	private String						text;
	// can have children or not
	private boolean						canHaveChildren;
	// is currently pulled down or up, true = pulled down
	private boolean						pull;
	private Dimension					preferredSize;
	// current font
	private Font							font;
	// current font metrics - speeds up preferredSize method response
	private FontMetrics				fm;


	// constructs new tree string element, sets pull state and 
	// "can have children" state
	public XFStringTreeElement (String text, boolean canHaveChildren, 
															boolean pull) {
		this. canHaveChildren = canHaveChildren;
		this.pull = canHaveChildren==true ? pull : false;
		this.text = text;
		// set the default font
		setFont(new Font("Helvetica", Font.PLAIN, 14));
	}

	/**
	 * Paints string from specified left center point
	*/
	public void paint (Graphics g, int x, int y) {
		g.drawString(text, x+2, 
									y+((fm.getAscent()+fm.getDescent())/2-fm.getDescent()));
	}

	/**
	 * Paints string with border
	*/
	public void paintSelected (Graphics g, int x, int y) {
		g.drawRoundRect(x, y-fm.getHeight()/2, 
										fm.stringWidth(text)+4, fm.getHeight(), 2, 2);
		paint(g, x, y);
	}

	/**
	 * Returns preferred size
	*/
	public Dimension preferredSize () {
		return preferredSize;
	}

	/**
	 * Returns true if string element can have children, false otherwise
	*/
	public boolean canHaveChildren () {
		return canHaveChildren;
	}

	/**
	 * Returns true if children are visible(pulled down) false otherwise
	*/
	public boolean isPulled () {
		return pull;
	}

	/**
	 * Sets string element to specified state, true = pulled down
	*/
	public void pull (boolean state) {
		if (!canHaveChildren) return;
		pull = state;
	}

	public void setFont (Font font) {
		this.font = font;
		fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
		recalculateSize();
	}

	public String getText () {
		return text;
	}

	public void setText (String text) {
		this.text = text;
		recalculateSize();
	}

	/**
	 * Recalculates preferred size. We assume that fm is not null.
	*/
	private void recalculateSize () {
		preferredSize = new Dimension(fm.stringWidth(text)+2*XBORDER,
																		fm.getHeight());
	}

}
