/**
* Xelfi programming environment
*/

package xelfi.browser.document;

import xelfi.editor.*;
import java.util.Vector;
import java.io.IOException;
import java.io.InputStream;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.io.DataInputStream;
import java.io.DataOutputStream;

/**
* This class
*
* @version 1.0
* @author Jan Jancura
*/
public class BrMethod extends BrMember
{
    private String[]    parameters;
    private String[]    throwss;

    public BrMethod(
                MyPosition  pPositionB,
                MyPosition  pPositionE,
                MyPosition  pCommentPositionB,
                MyPosition  pCommentPositionE,
                BrObject    pObject,
                String      pName,
                XFile       pXFile,
                Modifiers   modif,
                String      pType,

                String      pParameters,
                String[]    pThrows)
    {
        super(pPositionB,pPositionE,pCommentPositionB,pCommentPositionE,
            pObject,pName,pXFile,
            modif,pType);

        if( pThrows == null ) throwss = new String[0];
        else throwss = pThrows;

        initParameters(pParameters);
        initFlags();

        return;
    }
    /**
     * This method
     *
     * @param
     * @return
     * @exception
     */
    public BrMethod(
                MyPosition  pPositionB,
                MyPosition  pPositionE,
                MyPosition  pCommentPositionB,
                MyPosition  pCommentPositionE,
                BrObject    pObject,
                String      pName,
                XFile       pXFile,
                Modifiers   modif,
                String      pType,

                String      pParameters,
                String      pThrows)
    {
        super(pPositionB,pPositionE,pCommentPositionB,pCommentPositionE,
            pObject,pName,pXFile,
            modif,pType);

        //throws
        Vector v = new Vector(10,10);
        StringTokenizer st = new StringTokenizer(pThrows,",\r\n",
            false);
        try
        {
            String s = st.nextToken().trim();
            if( s.startsWith("throws") )
                s = s.substring(7).trim();

            v.addElement(s);

            while( st.hasMoreTokens() )
            {
                v.addElement(st.nextToken().trim());
            }
        }catch(NoSuchElementException e)
        {}

        int i,k = v.size();
        throwss = new String[k];
        for(i=0;i<k;i++)
            throwss[i] = (String)v.elementAt(i);

        initParameters(pParameters);
        initFlags();

        return;
    }
    BrMethod(BrObject obj,DataInputStream is)
        throws IOException
    {
        super(obj,is);
        int i,k = is.readChar();
        parameters  = new String[k];
        for(i=0;i<k;i++)
            parameters[i] = is.readUTF();
        k = is.readChar();
        throwss  = new String[k];
        for(i=0;i<k;i++)
            throwss[i] = is.readUTF();
        initFlags();
        return;
    }
    void save(DataOutputStream os) throws EndOfTextException,
        IOException
    {
        super.save(os);
        int i,k = parameters.length;
        os.writeChar(k);
        for(i=0;i<k;i++)
        {
            os.writeUTF(parameters[i]);
        }
        k = throwss.length;
        os.writeChar(k);
        for(i=0;i<k;i++)
        {
            os.writeUTF(throwss[i]);
        }
        return;
    }
    private void initParameters(String pParameters)
    {
        Vector v = new Vector(10,10);

        // parameters
        StringTokenizer st = new StringTokenizer(pParameters,",\r\n",
            false);
        try
        {
            String s = st.nextToken().trim();
            if( s.startsWith("(") )
                s = s.substring(s.indexOf("(") + 1);

            v.addElement(s);

            while( st.hasMoreTokens() )
            {
                v.addElement(st.nextToken());
            }
        }catch(NoSuchElementException e)
        {}

        int i,k = v.size();
        parameters = new String[k];
        for(i=0;i<k;i++)
            parameters[i] = (String)v.elementAt(i);

        if( (k > 0) && parameters[k-1].endsWith(")") )
            parameters[k-1] = parameters[k-1].substring(0,
                parameters[k-1].lastIndexOf(')'));

        return;
    }
    private void initFlags()
    {
        StringBuffer buf = new StringBuffer();
        buf.append(modifiers.abstractM() ? 'a' : '-');
        buf.append(modifiers.finalM() ? 'f' : '-');
        buf.append(modifiers.nativeM() ? 'n' : '-');
        buf.append(modifiers.staticM() ? "St" : "--");
        buf.append(modifiers.synchronizedM() ? "Sy" : "--");
        buf.append(modifiers.access());
        buf.append(" ");
        flags = new String(buf);

        return;
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public String[] getParameters()
    {
        return parameters;
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public String[] getThrows()
    {
        return throwss;
    }

   /**
     * This method return name with modifiers and other
     * parameters.
     * @param bs Actual settings.
     * @param depth If this component listbox is hierarched
     * this parameter stores information about components depth
     * in this tree.
     * @return Full name
     */
    public String getFullName(BrSettings bs,int depth)
    {
        StringBuffer s = new StringBuffer();
        if( bs.MemShowModif )
        {
            s.append(flags);
        }
        if( bs.MemShowType )
        {
            s.append(type);
            int i,k = bs.MemIndent - type.length();
            k = k > 0 ? k : 1;
            for(i=0;i<k;i++)
                s.append(' ');
        }
        s.append(name).
        append('(');
        int i,k = parameters.length;
        for(i=0;i<(k-1);i++)
            s.append(parameters[i]).append(",");
        if( k>0 )
            s.append(parameters[k-1]);
        s.append(')');
        return new String(s);
    }

   /**
     * This method generates source text of this method.
     * @return The text.
     */
    public String getText()
    {
        StringBuffer b = new StringBuffer();
        switch( modifiers.access() )
        {
        case 0: b.append("public ");break;
        case 2: b.append("private ");break;
        case 3: b.append("protected ");break;
        }
        b.append(modifiers.abstractM() ? "abstract " : "").
        append(modifiers.finalM() ? "final " : "").
        append(modifiers.nativeM() ? "native " : "").
        append(modifiers.staticM() ? "static " : "").
        append(modifiers.synchronizedM() ? "synchronized " : "").

        append(type).
        append(" ").
        append(name).
        append('(');
        int i,k = parameters.length;
        for(i=0;i<(k-1);i++)
            b.append(parameters[i]).append(",");
        if( k>0 )
            b.append(parameters[k-1]);
        b.append(')');

        //throws
        k = throwss.length;
        if( k>0 )
            b.append(" throws ");
        for(i=0;i<(k-1);i++)
            b.append(throwss[i]).append(", ");
        if( k>0 )
            b.append(throwss[k-1]);

        //append("\n{");
        return new String(b);
    }

    void open(Document d)
    {
        positionBegin.open(d);
        positionEnd.open(d);
        if( positionBodyEnd != null )
            positionBodyEnd.open(d);
        if( commentPositionB != null )
        {
            commentPositionB.open(d);
            commentPositionE.open(d);
        }
    }
    void close()
    {
        positionBegin.close();
        positionEnd.close();
        if( positionBodyEnd != null )
            positionBodyEnd.close();
        if( commentPositionB != null )
        {
            commentPositionB.close();
            commentPositionE.close();
        }
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public void deleteMember()
        throws IOException,InvalidPositionException,NotAllowedException,
            EndOfTextException
    {
        Block b;
        Source s = new Source(getXFile());
        (new Block(getBegin(),getBodyEnd())).delete();
        if( commentPositionB != null )
            (new Block(commentPositionB.getPosition().previous(3),
                commentPositionE.getPosition())).delete();
        object.removeComponent(this);
        return;
    }

   /**
     * This method allows adding a new member behind
     * this one. It generates source code too.
     * @param newMember Member to add.
     * @exception IOException
     * @exception NotAllowedException Occures, when you adds text
     * to the form.
     * @exception EndOfText Exception
     */
    public void addMember(BrMember newMethod)
        throws IOException,NotAllowedException,EndOfTextException
    {
        Position p,pom;
        Source s = new Source(getXFile());
        pom = getBodyEnd();
        p = pom.insert("\n    " +
            newMethod.getText() + "\n    {\n    }\n");

        object.addComponentBehind(newMethod,this);
        newMethod.setBegin(new MyPosition(p.beginOfLine().down()
            .next(4)));
        newMethod.setEnd(new MyPosition(p.beginOfLine().down(2).
            endOfLine()));
        newMethod.setBodyEnd(new MyPosition(pom.up().endOfLine()));
        setEnd(new MyPosition(p));
        return;
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public void changeMember(BrMember newMethod)
        throws IOException,NotAllowedException,EndOfTextException
    {
        Position p,pom;
        Source s = new Source(getXFile());
        (new Block(getBegin(),getEnd())).delete();
        pom = getEnd();
        p = pom.insert(newMethod.getText() + "\n    {");

        object.addComponentBehind(newMethod,this);
        object.removeComponent(this);

        newMethod.setBegin(new MyPosition(p));
        newMethod.setEnd(new MyPosition(p.beginOfLine().down().
            endOfLine()));
        newMethod.setBodyEnd(new MyPosition(getBodyEnd()));
        return;
    }
    public String getHint()
        throws IOException
    {
        if( commentPositionB == null )
        {
            StringBuffer b = new StringBuffer();
            b.append("This method ... \n\n");
            int i,k = parameters.length;
            for(i=0;i<k;i++)
                b.append("@param ").append(parameters[i]).append('\n');

            k = throwss.length;
            for(i=0;i<k;i++)
                b.append("@exception ").append(throwss[i]).append('\n');

            if( (!type.equals("void")) && (type.length()>0) )
                b.append("@return ").append(type).append('\n');

            b.append("@see\n");

            return new String(b);
        }else
        {
            Source src = new Source(getXFile());
            InputStream is = (new Block(commentPositionB.getPosition(),
                commentPositionE.getPosition())).
                getInputStream();
            StringBuffer b = new StringBuffer();
            int c;
            while( (c = is.read()) >= 0 ) b.append((char)c);
            String s = new String(b);

            return hintToString(s);
        }
    }
}







