/**
* Xelfi
*/

package xelfi.design.components.dialogs;

import java.awt.*;
import xelfi.design.components.*;
import xelfi.design.util.XFDesignConstants;
import xelfi.design.util.GridInfo;
import xelfi.top.TopLevel;
import xelfi.util.Debug;

/**
 * GridSetThread class is here only to get working modal XFGridSetDialog 
 * with JDK 1.02 under Win32 (Sunsoft claimed that this bug is fixed in JDK 1.02
 * but not under Win32 - the show() method of the Dialog class is still non-blocking)
 *
 * @version 	1.00, 25 Feb 1997
 * @author 	Ian Formanek
 */

public class GridSetThread extends Thread {
	private Frame parent;
	private XelfiDesignComponent xdc;
	private int propNum;
	private Container compParent;
	
	/**
	 * Constructor to set parameters for the thread and for the font dialog
	 * @param	name				the name of the thread
	 * @param	aParent			parent of the dialog
	 * @param	aXdc				Xelfi component whose Font propety to set
	 * @param	aPropNum		index of the Font property
	 * @param	aCompParent	parent of the component which validate() method is called 
	 */
	public GridSetThread(String name, Frame aParent, XelfiDesignComponent aXdc, 
										int aPropNum) {
		super(name);		
		parent = aParent;
		xdc = aXdc;
		propNum = aPropNum;
	}

	/**
	 * A method which actually runs the XFGridSetDialog to set grid.
	 * It first gets current grid settings from the component and then invokes the 
	 * XFGridSetDialog. Since the method show() immediately returns, it starts an "infinite"
	 * loop which checks whether the dialog has finished. Then, if user clicked OK, the 
	 * grid property of the component is set according to the grid set-up in the dialog.
	 */
	public void run() {
		Object of = null;
		try {
			of = xdc.getProperty(propNum);
			} catch (XFPropertyIndexOutOfBoundsException e) {
		}
		if (of instanceof GridInfo) {
			XFGridSetDialog gsd = new XFGridSetDialog(parent, (GridInfo) of);
			gsd.show();
			// for the dialog to run at reasonable speed
			// we must set to this loop minimal priority
			setPriority(MIN_PRIORITY);
			do { } while (gsd.finished != true);			
			if (gsd.result == XFGridSetDialog.RES_OK) {
				// set the grid property and catch exceptions
				try {
					int ret = xdc.setProperty(propNum, gsd.gi);
					TopLevel.getInspector().propertiesChanged();
				} catch (XFPropertyBadValueTypeException e) {	
					Debug.debugException(e);
				} catch (XFPropertyIndexOutOfBoundsException e) {
					Debug.debugException(e);
				}
			}
		}
		else Debug.debug(Debug.ERROR, "Property number: "+ propNum + " of component: "+ xdc +" is not of type GridInfo");
	}
}