/**
* Xelfi
*/

package xelfi.design.components.dialogs;

import xelfi.awt.ColorGrid;
import java.awt.*;

/**
 * @version 	1.00, 02 Dec 1996
 * @author 	Petr Hamernik
 */


public class XFColorDialog extends Dialog {

	public static final int RES_OK = 1;
	public static final int RES_CANCEL = 2;

	public boolean finished = false;
	
	public int result = 0;

    private Scrollbar scrR;
    private Scrollbar scrG;
    private Scrollbar scrB;
    private TextField editR;
    private TextField editG;
    private TextField editB;
    private Button OKBut;
    private Button CancelBut;
    private Label LabelR;
    private Label LabelG;
    private Label LabelB;
    private ColorGrid colGrid;
    private ColorPreview colPrev;

    public XFColorDialog(Frame parent, Color col) {

	    super(parent, "Color Selection", true);

		setResizable(false);

        setLayout(null);
        addNotify();
        resize(insets().left + insets().right + 390, insets().top + insets().bottom + 165);
        scrR= new Scrollbar(Scrollbar.HORIZONTAL, 255,10,0,255);
        add(scrR);
        scrR.reshape(insets().left + 54,insets().top + 16,153,13);
        scrG= new Scrollbar(Scrollbar.HORIZONTAL, 255,10,0,255);
        add(scrG);
        scrG.reshape(insets().left + 54,insets().top + 44,153,13);
        scrB= new Scrollbar(Scrollbar.HORIZONTAL, 255,10,0,255);
        add(scrB);
        scrB.reshape(insets().left + 54,insets().top + 72,153,13);
        editR=new TextField(7);
        add(editR);
        editR.reshape(insets().left + 222,insets().top + 14,72,24);
        editG=new TextField(7);
        add(editG);
        editG.reshape(insets().left + 222,insets().top + 41,72,24);
        editB=new TextField(7);
        add(editB);
        editB.reshape(insets().left + 222,insets().top + 68,72,24);

        colPrev = new ColorPreview();
        colPrev.reshape(insets().left + 300,insets().top + 14,78,78);
        add(colPrev);

        OKBut=new Button("Ok");
        add(OKBut);
        OKBut.reshape(insets().left + 220,insets().top + 122,60,24);
        CancelBut=new Button("Cancel");
        add(CancelBut);
        CancelBut.reshape(insets().left + 300,insets().top + 122,60,24);
        LabelR=new Label("Red");
        add(LabelR);
        LabelR.reshape(insets().left + 9,insets().top + 16,45,16);
        LabelG=new Label("Green");
        add(LabelG);
        LabelG.reshape(insets().left + 9,insets().top + 44,45,16);
        LabelB=new Label("Blue");
        add(LabelB);
        LabelB.reshape(insets().left + 9,insets().top + 72,45,16);

        colGrid = new ColorGrid();
        add(colGrid);
        colGrid.move(insets().left + 30,insets().top + 102);
		colGrid.setGridOrdering(ColorGrid.GO_8x2);
		colGrid.resize(colGrid.preferredSize());		
		colGrid.bgEnable(false);
		colGrid.setFg(0);
						
        int r = col.getRed();
        int g = col.getGreen();
        int b = col.getBlue();
        editR.setText(new Integer(r).toString());
        editG.setText(new Integer(g).toString());
        editB.setText(new Integer(b).toString());
        scrR.setValue(r);
        scrG.setValue(g);
        scrB.setValue(b);
        colPrev.setColor(r, g, b);
    }

    public synchronized void show() {
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolkit.getScreenSize();
		Dimension dialogSize = size();
		move((screenSize.width-dialogSize.width)/2,(screenSize.height-dialogSize.height)/2);
		super.show();
    }

    public synchronized void wakeUp() {
    	notify();
    }

    public boolean handleEvent(Event event) {
    	if (event.target == scrR) {
    	    editRUpdate();
    	}
    	else if (event.target == scrG) {
    	    editGUpdate();
    	}
    	else if (event.target == scrB) {
    	    editBUpdate();
    	}
    	else
    	if (event.id == Event.WINDOW_DESTROY) {
    	    hide();
    	    dispose();
    	    return true;
    	}
		else
		if (event.id == ColorGrid.COL_GRID_CHANGE) {
			gridUpdate(colGrid.getFgColor());
			return true;
		}
		return super.handleEvent(event);
    }


	public boolean action(Event event, Object arg) {
		if (event.target == OKBut) {
			result = RES_OK;
			finished = true;
			hide();
			dispose();
		}
		else if (event.target == CancelBut) {
			result = RES_CANCEL;
			finished = true;
			hide();
			dispose();
		}
		else
		if (event.target == editR) {
            scrRUpdate();
		}
		else if (event.target == editG) {
            scrGUpdate();
		}
		else if (event.target == editB) {
            scrBUpdate();
		}

		return true;
	}

	public Color getSelColor() {
		return new Color(scrR.getValue(), scrG.getValue(), scrB.getValue());
	}

    private void editRUpdate() {
        editR.setText(new Integer(scrR.getValue()).toString());
        colPrevUpdate();
    }

    private void editGUpdate() {
        editG.setText(new Integer(scrG.getValue()).toString());
        colPrevUpdate();
    }

    private void editBUpdate() {
        editB.setText(new Integer(scrB.getValue()).toString());
        colPrevUpdate();
    }

    private void scrRUpdate() {
		Integer tmp = null;
		try {
			tmp = new Integer(editR.getText());
		} catch (NumberFormatException e) {	}
		if (tmp != null) {
            int v = tmp.intValue();
            if (v < 0) v = 0;
            if (v > 255) v = 255;
            editR.setText(new Integer(v).toString());
            scrR.setValue(v);
            colPrevUpdate();
		}
    }

    private void scrGUpdate() {
		Integer tmp = null;
		try {
			tmp = new Integer(editG.getText());
		} catch (NumberFormatException e) {	}
		if (tmp != null) {
            int v = tmp.intValue();
            if (v < 0) v = 0;
            if (v > 255) v = 255;
            editG.setText(new Integer(v).toString());
            scrG.setValue(v);
            colPrevUpdate();
		}
    }

    private void scrBUpdate() {
		Integer tmp = null;
		try {
			tmp = new Integer(editB.getText());
		} catch (NumberFormatException e) {	}
		if (tmp != null) {
            int v = tmp.intValue();
            if (v < 0) v = 0;
            if (v > 255) v = 255;
            editB.setText(new Integer(v).toString());
            scrB.setValue(v);
            colPrevUpdate();
		}
    }

    private void colPrevUpdate() {
        colPrev.setColor(scrR.getValue(), scrG.getValue(), scrB.getValue());
    }

    private void gridUpdate(Color c) {
        int r = c.getRed();
        int g = c.getGreen();
        int b = c.getBlue();
        editR.setText(new Integer(r).toString());
        editG.setText(new Integer(g).toString());
        editB.setText(new Integer(b).toString());
        scrR.setValue(r);
        scrG.setValue(g);
        scrB.setValue(b);
        colPrev.setColor(r, g, b);
    }

}

class ColorPreview extends Canvas {
    Color c = null;
	ColorPreview() {
        c = new Color(255,255,0);
	}

	public void setColor(Color col) {
        c = col;
        repaint();
	}

	public void setColor(int r, int g, int b) {
	    c = new Color(r, g, b);
	    repaint();
	}

	public void paint(Graphics g) {
		Dimension d = size();
		g.setColor(getBackground());
		g.draw3DRect(0, 0, d.width - 1, d.height - 1, true);
		g.draw3DRect(2, 2, d.width - 5, d.height - 5, false);
		g.setColor(c);
		g.fillRect(4, 4, d.width - 9, d.height - 9);
	}
}

