/**
* Xelfi
*/

package xelfi.design.components.dialogs;

import java.awt.*;

/**
 * XFArrayStringDialog is a dialog for setting XFArrayString property
 *
 * @version 	1.01, 06 Oct 1996
 * @author 	Petr Hamernik
 */

public class XFStringArrayDialog extends Dialog {
	public static final int RES_OK = 1;
	public static final int RES_CANCEL = 2;

	public boolean finished = false;
	public int result = 0;

    private TextArea list;
    private Button OKBut;
    private Button CancelBut;

	public XFStringArrayDialog(Frame parent) {
	    super(parent, "XFArrayStringDialog dialog", true);
		insertComp();
	}
		
	public XFStringArrayDialog(Frame parent, String aStr) {
	    super(parent, "XFArrayStringDialog dialog", true);
		insertComp();

		if (aStr != null)
			setString(aStr);
	}
	
	public XFStringArrayDialog(Frame parent, String[] aStr) {
	    super(parent, "XFArrayStringDialog dialog", true);
		insertComp();

		if (aStr != null)
			setStrings(aStr);
	}
	
	public void insertComp() {
        setLayout(null);
        addNotify();
        resize(insets().left + insets().right + 475, insets().top + insets().bottom + 298);
        list=new TextArea(12,32);
        add(list);
        list.reshape(insets().left + 9,insets().top + 6,324,258);
        OKBut=new Button("OK");
        add(OKBut);
        OKBut.reshape(insets().left + 351,insets().top + 192,108,32);
        CancelBut=new Button("Cancel");
        add(CancelBut);
        CancelBut.reshape(insets().left + 351,insets().top + 232,108,32);

    	setResizable(false);
		pack();
		}

    public synchronized void show() {
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolkit.getScreenSize();
		Dimension dialogSize = size();
		move((screenSize.width-dialogSize.width)/2,(screenSize.height-dialogSize.height)/2);
		super.show();
    }

	public boolean action(Event event, Object arg) {
		if (event.target == OKBut) {
			result = RES_OK;
			finished = true;
			hide();
			dispose();
			}
		else if (event.target == CancelBut) {
			result = RES_CANCEL;
			finished = true;
			hide();
			dispose();
			}
		return true;
	}

	public void setStrings(String[] aStr) {
		for (int i = 0; i < aStr.length; i++)
			if (aStr[i] != null) {
				list.appendText(aStr[i]);
				if (i < aStr.length - 1)
					list.appendText("\n");
			}
	}

	public String[] getStrings() {
		int l = list.getRows();
		
		String[] tmp = new String[l];
		String text = list.getText();
		int start = 0;
		int end = 0;
		for (int i = 0; i < l; i++) {
			end = text.indexOf("\n", start);
			if (end == -1)
				end = text.length();
			if (start == end)
				tmp[i] = new String("");
			else
				tmp[i] = text.substring(start,end);
			if (end >= text.length()) {
				end = i;
				break;
			}
			start = end + 1;
		}
		if (tmp[end].length() == 0)
			end--;
		String[] ret = new String[end + 1];
		for (int i = 0; i <= end; i++)
			ret[i] = new String(tmp[i]);
		return ret;
	}
	
	public void clearStrings() {
		String s = "";
		list.replaceText(s, 0, list.getText().length());
	}
	
	public void setString(String aStr) {
		list.setText(aStr);
	}

	public String getString() {
		return list.getText();
	}


	public boolean handleEvent(Event event) {
		if (event.id == Event.WINDOW_DESTROY)
			hide();
		return super.handleEvent(event);
		}

}
