/**
* Xelfi
*/

package xelfi.design.components.xawt;

/**
  *
  * @version 	2.12, 03 Apr 1997
  * @author 	Petr Hamernik, Ian Formanek
  *
  */

import java.awt.*;
import java.io.*;
import xelfi.design.components.*;
import xelfi.design.components.property.*;
import xelfi.design.forms.XelfiDesignForm;
import xelfi.design.util.XFDesignConstants;
import xelfi.design.util.XFDesignOptions;
import xelfi.design.util.XFDesignUtils;

public class XCheckbox extends XAWTPanel {
	private Checkbox check;
	private static int propertyCount = 10;
	private Object propDef[];
	private boolean propDefSet = false;
	private static int eventCount = 3;
	private String events[] = new String[eventCount];
	protected static Image icon = null;

	private boolean labeled = false;

	public XCheckbox() { 
		super(XAWTPanel.WITH_DRAGBORDER);
		setComponent(check = new Checkbox());
		propDef = new Object[propertyCount];
		for (int i=0; i<propertyCount; i++)
			propDef[i] = null;
		if (icon == null)
			icon = loadIcon("checkbox.gif");
	}

	protected void regenForm(int what) {
		if (parentForm != null)
			parentForm.regenerateCode(this, what);
	}

	public int setDesignName(String aName) {
		if (!labeled) {
			check.setLabel(aName); 
			if (autoSize)
				setDesignSize(0,0);
		}
		super.setDesignName(aName);
		if (!labeled) return XFDesignConstants.RSP_OTHER_PROP;
		else return XFDesignConstants.RSP_OK;
	}

	public String[] neededImports() {
		String[] ret = new String[1];
		ret[0] = "java.awt.Checkbox";
		return ret;
	}

	public Image getIcon() {
		return icon; 
	}
	
	private boolean propertyDefault(int i) throws XFPropertyIndexOutOfBoundsException {
		if (propDefSet)
			propertiesSetDefault();
		if (propDef[i] == null)
			return false;
		switch (i) {
			case 0: return ((Color) propDef[i]).equals(check.getBackground());
			case 1: return ((Color) propDef[i]).equals(check.getForeground());
			case 2: return (((Boolean) propDef[i]).booleanValue() == check.isEnabled());
			case 3: return (((Boolean) propDef[i]).booleanValue() == bVisible);
			case 4: return ((Font) propDef[i]).equals(check.getFont());
			case 5: return false;
			case 6: return (((Boolean) propDef[i]).booleanValue() == autoSize);
			case 7: return (((Boolean) propDef[i]).booleanValue() == parentColor);
			case 8: return (((Boolean) propDef[i]).booleanValue() == parentFont);
			case 9: return (((Boolean) propDef[i]).booleanValue() == check.getState());
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	private void propertiesSetDefault() {
		if (propDefSet)
			return;
		propDefSet = true;
		if (check.getBackground() != null)
				propDef[0] = new Color(check.getBackground().getRGB());
		if (check.getForeground() != null)
				propDef[1] = new Color(check.getForeground().getRGB());
		propDef[2] = new Boolean(check.isEnabled());
		propDef[3] = new Boolean(bVisible);
		if (check.getFont() != null)
			propDef[4] = new Font(check.getFont().getName(),check.getFont().getStyle(),check.getFont().getSize());
		propDef[5] = new String("");
		propDef[6] = new Boolean(autoSize);
		propDef[7] = new Boolean(parentColor);
		propDef[8] = new Boolean(parentFont);
		propDef[9] = new Boolean(check.getState());
	}
		
	public int getPropertyCount() { return propertyCount; }
	
	public Object getProperty(int index) throws XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: return check.getBackground();
			case 1: return check.getForeground();
			case 2: return (check.isEnabled()) ? Boolean.TRUE : Boolean.FALSE;
			case 3: return (bVisible) ? Boolean.TRUE : Boolean.FALSE;
			case 4: return check.getFont();
			case 5: return check.getLabel();
			case 6: return (autoSize) ? Boolean.TRUE : Boolean.FALSE;
			case 7: return (parentColor) ? Boolean.TRUE : Boolean.FALSE;
			case 8: return (parentFont) ? Boolean.TRUE : Boolean.FALSE;
			case 9: return (check.getState()) ? Boolean.TRUE : Boolean.FALSE;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}
			
	public String[] getPropertyNames() { 
		String list[] = new String[propertyCount];
		list[0] = "Background Color";
		list[1] = "Foreground Color";
		list[2] = "Enabled";
		list[3] = "Visible";
		list[4] = "Font";
		list[5] = "Label";
		list[6] = "Auto Size";
		list[7] = "Parent Color";
		list[8] = "Parent Font";
		list[9] = "State";
		return list;
	}
		
	public String getPropertyString(int index) throws 
		XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: 
			case 1: return XColorPropertyType.viewString(getProperty(index));
			case 2:
			case 3: return XBooleanPropertyType.viewString(getProperty(index));
			case 4: return XFontPropertyType.viewString(getProperty(index));
			case 5: return XStringPropertyType.viewString(getProperty(index));
			case 6: 
			case 7: 
			case 8:
			case 9: return XBooleanPropertyType.viewString(getProperty(index));
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int getPropertyViewerType(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			case 0: 
			case 1: return XFDesignConstants.PVT_SPECIAL;
			case 2:
			case 3: return XFDesignConstants.PVT_CHECK;
			case 4: return XFDesignConstants.PVT_SPECIAL;
			case 5: return XFDesignConstants.PVT_INPUT_STRING;
			case 6:
			case 7:
			case 8:
			case 9: return XFDesignConstants.PVT_CHECK;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValueNames(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object[] getPropertyValueValues(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int setProperty(int index, Object value) throws 	
		XFPropertyIndexOutOfBoundsException,
		XFPropertyBadValueTypeException 
	{ 
		int ret = XFDesignConstants.RSP_OK;
		int regen = XFDesignConstants.GEN_SETPROPS;
		if (!propDefSet)
			propertiesSetDefault();
		switch (index) { 
			case 0: if (!(value instanceof Color)) throw new XFPropertyBadValueTypeException();
					check.setBackground((Color)value);
					ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
					check.hide();
					check.show();
					if (parentColor) {
						parentColor = false;
						ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					}
					break;
			case 1:	if (!(value instanceof Color)) throw new XFPropertyBadValueTypeException();
					check.setForeground((Color)value);
					ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
					check.hide();
					check.show();
					if (parentColor) {
						parentColor = false;
						ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					}
					break;
			case 2:	if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					check.enable(((Boolean)value).booleanValue());
					break;
			case 3:	if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					bVisible = ((Boolean)value).booleanValue();
					break;
			case 4:	if (!(value instanceof Font)) throw new XFPropertyBadValueTypeException();
					check.setFont((Font)value); 
					ret = ret | XFDesignConstants.RSP_REPAINT_COMP | XFDesignConstants.RSP_OTHER_PROP;
					if (parentFont) {
						parentFont = false;
						if (autoSize) {
							setDesignSize(0,0);
							regen = regen | XFDesignConstants.GEN_RESHAPE;
						}
					}
					repaintComponent();
					break;
			case 5:	if (!(value instanceof String)) throw new XFPropertyBadValueTypeException();
					check.setLabel((String)value);
					labeled = true; // so that caption of a check does not change with its name no more
					regen = regen | XFDesignConstants.GEN_INIT;
					if (autoSize) {
						setDesignSize(0,0);
						regen = regen | XFDesignConstants.GEN_RESHAPE;
						ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
						ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					}
					repaintComponent();
					break;
			case 6:	if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					setAutoSize(((Boolean)value).booleanValue());
					if (autoSize) {
						repaintComponent();
						setDesignSize(0,0);
						ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					}
					regen = regen | XFDesignConstants.GEN_RESHAPE;
					ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
					break;
			case 7:	if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					setParentColor(((Boolean)value).booleanValue());
					if (parentColor)
						ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
					break;
			case 8:	if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					setParentFont(((Boolean)value).booleanValue());
					if (parentFont)
						ret = ret | XFDesignConstants.RSP_REPAINT_COMP;
					if (autoSize)
					 	ret = ret | XFDesignConstants.RSP_OTHER_PROP;
					break;
			case 9:	if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					check.setState(((Boolean)value).booleanValue());
					break;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
		if (parentForm != null)
			parentForm.regenerateCode(this, regen);
		
		return ret;
	}
			
	public void specialPropertyInput(Frame parent, int index) throws 	
		XFPropertyIndexOutOfBoundsException, 
		XFPropertyNoSpecialInputException { 
		if ((index < 0) || (index >= propertyCount)) throw new XFPropertyIndexOutOfBoundsException();
		switch (index) { 
			case 0:
			case 1: XColorPropertyType.invokeSpecialInput(parent, this, index);
						break;
			case 4: XFontPropertyType.invokeSpecialInput(parent, this, index);
						break;
			default: throw new XFPropertyNoSpecialInputException();
		}
	}

	public int getEventCount() {
		return eventCount;
	}

	public String[] getEventNames() {
		String list[] = new String[eventCount];
		list[0] = "OnClick";
		list[1] = "OnGotFocus";
		list[2] = "OnLostFocus";
		return list;
	}

	public String getEvent(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		if ((index>=0) && (index<eventCount)) {
			if (events[index] == null)
				return null;
			else
				return new String(events[index]); 
		}
		else throw new XFPropertyIndexOutOfBoundsException();
	}		
		
	public void setEvent(int index, String value) throws 
		XFPropertyIndexOutOfBoundsException {
		if ((index>=0) && (index<eventCount))
			events[index] = value;
		else throw new XFPropertyIndexOutOfBoundsException();
	}		

	public String getClassName() {
		return "Checkbox";
	}
	
	public String generateCode(int what) {
		String gen = "";
		switch (what) {
			case XFDesignConstants.GEN_VARIABLE: 
				gen = getClassName()+" "+ name + ";\n";
				break;
			case XFDesignConstants.GEN_INIT: 
				gen = name + " = new " + getClassName() + "(\"" + check.getLabel() + "\");\n";
				break;
			case XFDesignConstants.GEN_SETPROPS:
				if (!propDefSet)
					propertiesSetDefault();
				for (int i = 0; i < propertyCount; i++) {
					if (!propertyDefault(i)) {
						switch(i) {
							case 0: if (!parentColor)
										gen = gen + name + ".setBackground(" +
											XFDesignUtils.genColor(check.getBackground()) + ");\n";
									break;
							case 1: if (!parentColor)
										gen = gen + name + ".setForeground(" +
											XFDesignUtils.genColor(check.getForeground()) + ");\n";
									break;
							case 2: gen = gen + name + (check.isEnabled() ? ".enable();\n" : ".disable();\n");
									break;
							case 3: gen = gen + name + ((bVisible) ? ".show();\n" : ".hide();\n");
									break;
							case 4: if (!parentFont) {
										gen = gen + name + ".setFont(" + 
											XFDesignUtils.genFont(check.getFont())+");\n";
									}
									break;
							case 9: gen = gen + name + ".setState(" + (check.getState()) + ");\n";
									break;
						}
					}
				}
				break;			
			case XFDesignConstants.GEN_RESHAPE:
				if (((XelfiVisualContainer) getParent()).getDesignLayout().allowsDragging()) {
					if (!autoSize) 
						if (getParent() instanceof XelfiDesignForm)
							gen = name + ".reshape(insets.left+"+position.x+", insets.top+"+position.y+","+size.width+","+size.height+");\n";
						else 
							gen = name + ".reshape("+position.x+","+position.y+","+size.width+","+size.height+");\n";
					else {
						gen = name + ".resize(" + name + ".preferredSize());\n";
						if (getParent() instanceof XelfiDesignForm)
							gen = gen + name + ".move(insets.left+"+position.x+", insets.top+"+position.y+");\n";
						else
							gen = gen + name + ".move("+position.x+","+position.y+");\n";
					}
				}
				break;
		}
		return gen;
	}

	public String generateEventCondition(int index) throws XFPropertyIndexOutOfBoundsException {
		String evt = XFDesignOptions.eventVarName;
		switch (index) { 
			case 0: return evt +".id == Event.ACTION_EVENT && "+evt+".target == "+getName();
			case 1: return evt +".id == Event.GOT_FOCUS && "+evt+".target == "+getName();
			case 2: return evt +".id == Event.LOST_FOCUS && "+evt+".target == "+getName();
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public void loadFromStream(DataInput stream) throws IOException {

		super.loadFromStream(stream);

		if (!propDefSet)
			propertiesSetDefault();

		for (int i = 0; i < propertyCount; i++) {
			if (stream.readBoolean()) {
				propDef[i] = null;
				if (i == 5)
					check.setLabel(name); // default label = component name
			}
			else switch (i) {
				case 0: check.setBackground(XColorPropertyType.loadFromStream(stream)); break;
				case 1: check.setForeground(XColorPropertyType.loadFromStream(stream)); break;
				case 2: check.enable(XBooleanPropertyType.loadBFromStream(stream)); break;
				case 3: bVisible = XBooleanPropertyType.loadBFromStream(stream); break;
				case 4: check.setFont(XFontPropertyType.loadFromStream(stream)); break;
				case 5: check.setLabel(XStringPropertyType.loadFromStream(stream)); break;
				case 6: setAutoSize(XBooleanPropertyType.loadBFromStream(stream)); break;
				case 7: setParentColor(XBooleanPropertyType.loadBFromStream(stream)); break;
				case 8: setParentFont(XBooleanPropertyType.loadBFromStream(stream)); break;
				case 9: check.setState(XBooleanPropertyType.loadBFromStream(stream)); break;
			}
		}
		for (int i = 0; i < eventCount; i++) {
			if (stream.readBoolean())
				parentForm.assignEvent(this, i, stream.readUTF());
		}
	}		
	
	public void saveToStream(DataOutput stream) throws IOException {

		super.saveToStream(stream);

		for (int i = 0; i < propertyCount; i++) {
			boolean b = propertyDefault(i);
			stream.writeBoolean(b);
			if (!b) {
				switch (i) {
					case 0: XColorPropertyType.saveToStream(check.getBackground(), stream); break;
					case 1: XColorPropertyType.saveToStream(check.getForeground(), stream); break;
					case 2: XBooleanPropertyType.saveToStream(check.isEnabled(), stream); break;
					case 3: XBooleanPropertyType.saveToStream(bVisible, stream); break;
					case 4: XFontPropertyType.saveToStream(check.getFont(), stream); break;
					case 5: XStringPropertyType.saveToStream(check.getLabel(), stream); break;
					case 6: XBooleanPropertyType.saveToStream(autoSize, stream); break;
					case 7: XBooleanPropertyType.saveToStream(parentColor, stream); break;
					case 8: XBooleanPropertyType.saveToStream(parentFont, stream); break;
					case 9: XBooleanPropertyType.saveToStream(check.getState(), stream); break;
				}
			}
		}

		for (int i = 0; i < eventCount; i++) {
			if ((events[i] != null) && (events[i].length() > 0)) {
				stream.writeBoolean(true);
				stream.writeUTF(events[i]);
			}
			else stream.writeBoolean(false);
		}
	}
	
}