/**
* Xelfi
*/

package xelfi.design.components.xawt;

import java.awt.*;
import java.io.*;
import xelfi.design.components.*;
import xelfi.design.components.property.*;
import xelfi.design.util.XFDesignConstants;
import xelfi.design.forms.XelfiDesignForm;
import xelfi.util.XelfiPath;
import xelfi.util.Debug;
import xelfi.top.*;
import xelfi.design.inspector.*;
  
/**
 * Xelfi non-visual component representing main menu folder - menu bar.
 * 
 * @version		1.22  25 Feb 1997
 * @author		Dafe, Ian Formanek
 */
public class XMenuBar extends MenuBar implements XelfiDesignComponent {

  // count of properties
  private static final int      propertyCount = 1;

  // menu icon
  private Image                 icon;
  private String		            name = "";
	// parent font property	
	private boolean			          parentFont;
  // parent form reference
  private XelfiDesignForm       parentForm;
	// flag for first setting of form parent
	private boolean								firstParent;
	// initial menu items - we must hold references because we
	// must ensure they will have good form parents.
  private XMenuFolder 					initMenu;
	private XMenuItem 						initItem;

	/**
	 * Constructs XMenuBar object with no parentForm set.
 	*/
  public XMenuBar () {
    this(null);
  }

	/**
	 * Constructs XMenuBar object with parentForm as specified.
	*/
	public XMenuBar (XelfiDesignForm form) {
    firstParent = true;
    initMenu = new XMenuFolder("Empty");
    initItem = new XMenuItem("Item");
    initMenu.add(initItem);
    add(initMenu);
    setParentForm(form);
    loadIcon();
	}

  public void setParentForm(XelfiDesignForm form) {
    parentForm = form;
		if ((parentForm!=null) && (firstParent)) {
			Debug.debug(Debug.MENU, "setting MenuBar parent "+parentForm);
			initMenu.setParentForm(parentForm);
			initItem.setParentForm(parentForm);
			parentForm.setComponentName(initMenu, "");
			parentForm.setComponentName(initItem, "");
			firstParent = false;
		}
  }

  public XelfiDesignForm getParentForm () {
    return parentForm;
  }

	public int getPropertyCount () { 
		return propertyCount;
	}
	
	public Object getProperty (int index) 
      throws XFPropertyIndexOutOfBoundsException {
    switch (index) {
      case 0: return this;
/*      case 1: return getFont();
      case 2: return parentFont==true ? Boolean.TRUE : Boolean.FALSE; */
      default:	throw new XFPropertyIndexOutOfBoundsException();
		}
	}
			
	public String[] getPropertyNames () { 
		String list[] = new String[propertyCount];
     list[0] = new String("Menu");
/*		list[1] = new String("Font");
		list[2] = new String("ParentFont"); */
		return list;
	}
		
	public String getPropertyString (int index)
		throws XFPropertyIndexOutOfBoundsException { 
    switch (index) {
        case 0: return "[Items]";
/*			case 1: return XFontPropertyType.viewString(getProperty(index));
			case 2: return XBooleanPropertyType.viewString(getProperty(index)); */
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int getPropertyViewerType (int index) 
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
      case 0: return XFDesignConstants.PVT_SPECIAL;
/*      case 1: return XFDesignConstants.PVT_SPECIAL;
      case 2: return XFDesignConstants.PVT_CHECK; */
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValueNames (int index)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object[] getPropertyValueValues (int index)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValuesArray (int index) 
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int setProperty (int index, Object value) 
		throws XFPropertyIndexOutOfBoundsException,
						XFPropertyBadValueTypeException 
	{ 
//		int ret = XFDesignConstants.RSP_OK;
 		throw new XFPropertyIndexOutOfBoundsException();
//		return ret;
	}
			
	public void specialPropertyInput (Frame parent, int index) 
		throws XFPropertyIndexOutOfBoundsException, 
						XFPropertyNoSpecialInputException { 
    if ((index < 0) || (index >= propertyCount))
      throw new XFPropertyIndexOutOfBoundsException();
    switch (index) {
      case 0: invokeME(); // invoke menu editor
        break;
/*      case 1: XFontPropertyType.invokeSpecialInput(parent, this, index);
        break; */
      default: throw new XFPropertyNoSpecialInputException();
		}
	}

	public Image getIcon () {
		return icon;
	}

	private void loadIcon() {
		String sep = new String(System.getProperty("file.separator"));
		StringBuffer s = new StringBuffer(XelfiPath.get() + sep + 
			"design" + sep + "components" + sep + "xawt" + sep + "menubar.gif");
		icon = Toolkit.getDefaultToolkit().getImage(new String(s));
	}

	public int getEventCount() {
		return 0;
	}

	public String[] getEventNames() {
		return null;
	}

	public String getEvent(int index)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}		
		
	public void setEvent(int index, String value)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}		

	public String getName () { 
		return name;
	}

	public int setDesignName (String aName) { 
		name = aName;
		return XFDesignConstants.RSP_OK;
	}

	public String getClassName () { return "MenuBar"; }
	
	public String generateEventCondition(int index) throws XFPropertyIndexOutOfBoundsException {
		throw new XFPropertyIndexOutOfBoundsException();
  }
  
	public String[] neededImports() {
		String list[] = new String[1];
		list[0] = "java.awt.MenuBar";
		return list;
	}

	public String generateCode (int what) { 
		String gen = "";
		switch (what) {
			case XFDesignConstants.GEN_VARIABLE: 
				gen = getClassName()+" "+ name + ";\n";
				for (int i=0; i<countMenus(); i++)
					gen = gen+((XelfiDesignComponent)getMenu(i)).generateCode(XFDesignConstants.GEN_VARIABLE);
				break;
			case XFDesignConstants.GEN_INIT: 
				gen = name + " = new " + getClassName() + "();\n";
				for (int i=0; i<countMenus(); i++)
					gen = gen+((XelfiDesignComponent)getMenu(i)).generateCode(XFDesignConstants.GEN_INIT);
				break;
		}
		return gen;
	}

	public void loadFromStream (DataInput stream) 
		throws IOException {
		remove(0); // removes the initialised "Empty" menu
		name = stream.readUTF();		
//		setFont(XFontPropertyType.loadFromStream(stream));
//		parentFont = XBooleanPropertyType.loadBFromStream(stream);
		int count = stream.readInt();
		for (int i=0; i<count; i++) {
			XMenuFolder xmn = new XMenuFolder();
			xmn.setParentForm(parentForm);
			xmn.loadFromStream(stream);
			add(xmn);
		}
	}
	
	public void saveToStream (DataOutput stream)
		throws IOException {
		stream.writeUTF(name);
//		XFontPropertyType.saveToStream(getFont(), stream);
//		XBooleanPropertyType.saveToStream(parentFont, stream);
		int count = countMenus();
		stream.writeInt(count);
		for (int i=0; i<count; i++) {
			XMenuFolder xmn = (XMenuFolder)getMenu(i);
			xmn.saveToStream(stream);
		}
	}

	public boolean parentUpdate () {
		return false;
	}

	public void setSelection(int aSelection) {
	}

	public int getSelection() {
		return XFDesignConstants.SEL_UNSELECT;
	}

  /**
   * Private method that invokes Menu Editor.
  */
  private void invokeME () {
    // get the reference to ME
    XelfiMenuEditor me = TopLevel.getMenuEditor();
    if (me==null) {
      // strange - toplevel should not be null
			Debug.debug(Debug.MENU, "!! Menu Editor reference is null - can't show ME");
      return;
    }
    if (me.getMenu()!=this) {
      // set ourself into ME
      me.setMenu(this);
			// set our parent form as target for visually created menu
			if (parentForm instanceof Frame)
				me.setTarget((Frame)parentForm);
			else
				Debug.debug(Debug.MENU, "Error: parentForm of the Menubar is not a Frame");
    }
		// show or focus Menu Editor
    if (!((Component)me).isVisible()) {
      ((Component)me).show();
    } else {
      ((Component)me).requestFocus();
		}
  }

	/**
	 * Refreshes content of menu editor.
	*/
	private void refreshME () {
    // get the reference to ME
    XelfiMenuEditor me = TopLevel.getMenuEditor();
    if (me==null) return;
		me.refresh();
	}

}
