/**
* Xelfi
*/

package xelfi.design.forms;

import java.io.*;
import xelfi.editor.*;
import xelfi.design.util.XFDesignOptions;
import xelfi.util.Debug;

/**
* Class that encapsulates blocks for one event (leading header, body and trailing parenthesis)
*
* @version 	1.08, 06 Apr 1997 
* @author 		Ian Formanek
*/

class EventBlock {
	String ev_name;
	Block ev_header, ev_body, ev_end;
	ForbidEmptyBlock feb;
	static String DEFAULT_TEXT=XFDesignOptions.indent(2) + "// Here you can write your own handling code\n"+
			XFDesignOptions.indent(2) + "return true;\n";
	
	EventBlock(Block _header, Block _body, Block _end, String aEv_name) {
		ev_name = aEv_name;
		ev_header = _header;
		ev_body = _body;
		ev_end = _end;
		feb = new ForbidEmptyBlock(ev_body, " ");
	}
	
	public Position getBegin() { return ev_header.getBegin(); }

	void rename(String newName) {
		ev_name = newName;
		try {
			ev_header.set(XFDesignOptions.indent(1)+
				"public boolean "+ev_name+"(Event "+XFDesignOptions.eventVarName+") {\n", new GuardedStamp());
		} catch (NotAllowedException e) {
			throw new InternalError("Block generation error");
		}
	}

	public void delete() {
		GuardedStamp gs = new GuardedStamp();
		try {
			ev_header.delete(gs);
			feb.stopWatching();
			ev_body.delete(gs);
			ev_end.delete(gs);
		} catch (NotAllowedException e) {
			throw new InternalError("Block generation error");
		}
	}
	
	static EventBlock create(Position pos, String aEv_name) {
		try {
			Block _end = Block.text(pos,XFDesignOptions.indent(1)+
				"}\n\n", new GuardedStamp());
			Block _body = Block.text(_end.getBegin(), DEFAULT_TEXT, new GuardedStamp());
			Block _header = Block.text(_body.getBegin(), XFDesignOptions.indent(1)+
				"public boolean "+aEv_name+"(Event "+XFDesignOptions.eventVarName+") {\n", new GuardedStamp());
			return new EventBlock(_header, _body, _end, aEv_name);
		} catch (NotAllowedException e) {
			throw new InternalError("Block generation error");
		}
	}

	public void loadFromStream(DataInput stream) throws
	IOException {
		String uCode = stream.readUTF();
		if ("".equals(uCode)) uCode = DEFAULT_TEXT;
		try {
			feb.stopWatching();
			ev_body.set(uCode, new GuardedStamp());
			feb = new ForbidEmptyBlock(ev_body, " ");
		} catch (NotAllowedException e) {
			throw new InternalError("Block generation error");
		}
	}

	public void saveToStream(DataOutput stream) throws
	IOException {
		String uCode = ev_body.toString();
		if (EventBlock.DEFAULT_TEXT.equalsIgnoreCase(uCode)) stream.writeUTF("");
		else stream.writeUTF(uCode);
	}
}
