/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.util.Enumeration;
import java.util.NoSuchElementException;
/**
* Extention of enumeration that allows access to every line begin in block.
* Firstly a begin of block is returned. Than every begin of line that follows
* is returned till end of block is reached.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class LineEnumeration extends Block implements Enumeration {
  /** the position to return */
  private Position ret;

  /** takes info from block */
  LineEnumeration (Block b) {
    begin = b.getBegin ();
    end = b.getEnd ();
    ret = begin.less (end) ? begin : null;
  };
  
  public Object nextElement () {
    if (ret == null) throw new NoSuchElementException ();
    
    Position p = ret;
    try {
      ret = ret.beginOfLine ().down ();
      if (ret.greaterOrEqual (end)) ret = null;
    } catch (EndOfTextException e) {
      ret = null;
    };
    return p;
  };
  
  public boolean hasMoreElements () {
    return ret != null;
  };
};


