/**
* Xelfi Environment
*/

package xelfi.editor.keys;

/** Describes one key from configuration file.
*
* @author Jaroslav Tulach
* @version 1.0
*/
final class KeyCode extends Object {
  /** constants for modifiers */
  public static final int SHIFT = 0x01;
  public static final int CTRL = 0x02;
  public static final int ALT = 0x04;

  /** number of all posible states */
  public static final int STATE_COUNT = 8;

  /** which modifiers must be down */
  int mod;
  /** which modifiers will be ignored */
  int ignore;
  /** key code. Taken from java.awt.Event */
  int key;
  /** next key or null */
  KeyCode next;

  /**
  * @param m modifers that must be down
  * @param i ignored modifiers
  * @param k key code
  */
  KeyCode (int m, int i, int k) {
    mod = m;
    ignore = i;
    key = k;
  }

  /** Returns array of booleans of size STATE_COUNT.
  * That are true if the appropriate combination of modifiers
  * is enought to fire this key.
  */
  boolean[] sets () {
    boolean[] ar = new boolean [STATE_COUNT];
    for (int i = 0; i < STATE_COUNT; i++) {
      // i with ignore must be superset of mod
      ar[i] = (mod & ~ignore) == (i & ~ignore);
    }
    return ar;
  }

  /** Hash. Returns the key code.
  * @return the hash code
  */
  public int hashCode () {
    return
      key <= 256 && Character.isLowerCase ((char)key) ?
      (int) Character.toUpperCase ((char)key)
      :
      key;
  }

  /** Equals when the key code is same.
  * @return true if equals
  */
  public boolean equals (Object i) {
    try {
      return i instanceof KeyCode && hashCode () == i.hashCode ();
    } catch (Exception ex) {
      return false;
    }
  }

  /** Converts to string.
  */
  public String toString () {
    StringBuffer sb = new StringBuffer ();
    toSB (sb);
    return sb.toString ();
  }

  /** Prints to string buffer.
  * @param sb string buffer to print to
  */
  private void toSB (StringBuffer sb) {
    sb.append ('[');
    sb.append ("Mod: ");
    sb.append (mod);
    sb.append (" Ignore: ");
    sb.append (ignore);
    sb.append (" Key: ");
    sb.append (key);
    if (next != null) {
      sb.append (" Next: ");
      next.toSB (sb);
    }
    sb.append (']');
  }
}




















// Generated from substitution template 'Class' Blocks:Class:2,34,9,34; Class:13,11,20,11; 
