/**
* Xelfi programming environment
*/

package xelfi.templates;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Toolkit;

import xelfi.awt.XelfiTreeViewElement;

/**
* Implements XelfiTreeViewElement interface for the template.
*
* @version $Id$
* @autor Jaroslav Tulach
*/

class TemplateImpl extends Template implements XelfiTreeViewElement {
  // horizontal border
  private static final int	XBORDER = 10;
  // is currently pulled down or up, true = pulled down
  private boolean pull = true;

  // current font
  private static Font font = new Font("Helvetica", Font.PLAIN, 14);
  // current font metrics - speeds up preferredSize method response
  private static FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
  // the dimension
  private Dimension size;

  /** Forbids outclass construction of a template.
  * @param name the name of template
  * @param supr the super template
  * @param manager the manager for this template
  */
  TemplateImpl (String name, Template supr, TemplateManager manager) {
    super (name, supr, manager);
  };

  /** Creates basic template.
  * @param name the name of template
  */
  TemplateImpl (String name) {
    this (name, null, new PlainManager ());
  };
  
  /**
  * Paints string from specified left center point
  */
  public void paint (Graphics g, int x, int y) {
    g.drawString (
      name, x + 2,
      y+((fm.getAscent()+fm.getDescent())/2-fm.getDescent())
    );
  }
  
  /**
  * Paints string with border
  */
  public void paintSelected (Graphics g, int x, int y) {
    g.drawRoundRect(x, y-fm.getHeight()/2, 
                    fm.stringWidth(name)+4, fm.getHeight(), 2, 2);
    paint(g, x, y);
  }
  
  /**
  * Returns preferred size
  */
  public Dimension preferredSize () {
    if (size == null) size = new Dimension(fm.stringWidth(name)+2*XBORDER, fm.getHeight());
    return size;
  }
  
  /**
  * Returns true if string element can have children, false otherwise
  */
  public boolean canHaveChildren () {
    return hasSubTemplates != null;
  }
  
  /**
  * Returns true if children are visible(pulled down) false otherwise
  */
  public boolean isPulled () {
    return pull && (hasSubTemplates != null);
  }
  
  /**
  * Sets string element to specified state, true = pulled down
  */
  public void pull (boolean state) {
    pull = state && (hasSubTemplates != null);
  }
};








