/** Color setting class
* @author: Filip Dvorak
* @version: 1.0  // Feb, 27 - changed by Ian - removed import xelfi.top.*
*/

package xelfi.top;

import java.awt.*;
import java.util.*;
import xelfi.util.EnhProperties;
import xelfi.top.visual.ColorPanel;
import xelfi.xedit.JavaEditorFrame;

  /** This class keeps settings of colors. Other classes can retrieve color values from it.
    They can also registrate itself here and will be notified when the setting is changed. */
public class XFColor implements XelfiOptions
{
    // types of color items
    /** Constants used by ColorPanel which is in another package. */
  public static final int TYPE_BG = 1;
  public static final int TYPE_BOTH = 3;
  public static final int TYPE_FG = 2;

   /** Constants used by ColorPanel which is in another package. */
  public static final int BG = 0;
  public static final int FG = 1;

  private static final int NO_COLOR_ITEMS = 8;
    // for accessing concrete color

  public static final int

      /** String constant color */
    StringConst = 0,
      /** Comment color */
    Comment = 1,
      /** Oyher text color */
    OtherText = 2,
      /** Selected text color */
    SelectedText = 3,
      /** Generated text color */
    GeneratedText = 4,
      /** Current error line color */
    ErrorLine = 5,
      /** Current execution line color */
    CurrentLine = 6,
      /** Breakpoint line color */
    BreakpointLine = 7;

    // color item descriptors
  private static final String propname[] =
    { "StringConstant", "Comment", "NormalText", "SelecetedText", "GeneratedText", "ErrorLine",
      "CurrentLine", "BreakpointLine" };
  private static final String name[] =
      { "String constant", "Comment", "Normal text", "Selected text", "Generated text", "Error line",
        "Current line", "Breakpoint line" };
  private static final int type[] =
    { TYPE_FG, TYPE_FG, TYPE_BOTH, TYPE_BOTH, TYPE_BG, TYPE_BOTH, TYPE_BOTH, TYPE_BOTH };
  private static final int specialIndex[] =
    { 2, 2, -1, -1, 2, -1, -1, -1 };  // index to another color item

    // color default values
  private static Color colorValue[][] =
  {
    { null, new Color(0, 150, 0) },
    { null, Color.red },
    { Color.white, Color.black },
    { Color.lightGray, Color.black },
    { Color.pink, null },
    { Color.red, Color.white },
    { Color.blue, Color.white },
    { Color.red, Color.black },
  };

    // default colors - indexes into ColorGrid
  private static int colorGridIndex[][] =
  {
    { -1, 14 },
    { -1, 10 },
    { 11, 0 },
    { 6, 0 },
    { 9, -1 },
    { 10, 11 },
    { 1, 11 },
    { 10, 0 },
  };

  private static Vector calls = new Vector(4);

    /** Get Background color.
      @param id id of the color item
      @return current setting for background of this color
    */
  public static Color getBgColor(int id)
  {
    return colorValue[id][BG];
  }

    /** Get foreground color.
      @param id id of the color item
      @return current setting for foreground of this color
    */
  public static Color getFgColor(int id)
  {
    return colorValue[id][FG];
  }

    /** XelfiOptions implementation. Loads color values from the EnhProperties structure. */
  public void loadProperties(EnhProperties prop)
  {
    int i;
    for(i = 0; i < NO_COLOR_ITEMS; i++)
    {
      if((type[i] & TYPE_FG) != 0)
      {
        colorValue[i][FG] = prop.getColorProperty(propname[i] + "FG", colorValue[i][FG]);
        colorGridIndex[i][FG] = prop.getIntProperty(propname[i] + "FGIndex", colorGridIndex[i][FG]);
      }
      if((type[i] & TYPE_BG) != 0)
      {
        colorValue[i][BG] = prop.getColorProperty(propname[i] + "BG", colorValue[i][BG]);
        colorGridIndex[i][BG] = prop.getIntProperty(propname[i] + "BGIndex", colorGridIndex[i][BG]);
      }
    }

    notifyCallbacks();
  }

    /** XelfiOptions implementation. Stores color values into the EnhProperties structure. */
  public void storeProperties(EnhProperties prop)
  {
    int i;
    for(i = 0; i < NO_COLOR_ITEMS; i++)
    {
      if((type[i] & TYPE_FG) != 0)
      {
        prop.setColorProperty(propname[i] + "FG", colorValue[i][FG]);
        prop.setIntProperty(propname[i] + "FGIndex", colorGridIndex[i][FG]);
      }
      if((type[i] & TYPE_BG) != 0)
      {
        prop.setColorProperty(propname[i] + "BG", colorValue[i][BG]);
        prop.setIntProperty(propname[i] + "BGIndex", colorGridIndex[i][BG]);
      }
    }

  }

    /** XelfiOptions implementation */
  public Panel getPanel()
  {
    return new ColorPanel(name, type, specialIndex, colorValue, colorGridIndex);
  }

    /** XelfiOptions implementation */
  public String getTabName()
  {
    return "Colors";
  }

    /** XelfiOptions implementation */
  public void onOK(Panel p)
  {
    ((ColorPanel)p).saveValues();
    notifyCallbacks();

      //!!  toto bude v budoucnu patrit mezi ColorCallbacks  !!
    xelfi.design.util.XFDesignOptions.GuardedBlockBackground = getBgColor(GeneratedText);

    updateDesktop();
  }

    /** Forces all editor windows to redraw. */
  protected static void updateDesktop()
  {
    Enumeration eds = TopLevel.allEditors();
    while(eds.hasMoreElements()) ((JavaEditorFrame)eds.nextElement()).getSource().getDocument().redraw();
  }

    /** Notifies all registrated XFColorCallbacks that color settings has changed. */
  protected void notifyCallbacks()
  {
    int size = calls.size(), i;
    for(i = 0; i < size; i++) ((XFColorCallback)calls.elementAt(i)).colorsChanged();
  }

    /** Add new callback. XFColorCallback will be notified when color settings are changed.
      @param cb new callback that will be added. First it will be notified immediately after adding so that it
        could read color values for the first time.*/
  public static void addCallback(XFColorCallback cb)
  {
    calls.addElement(cb);
    cb.colorsChanged();

      //!!  toto bude v budoucnu patrit mezi ColorCallbacks  !!
    xelfi.design.util.XFDesignOptions.GuardedBlockBackground = getBgColor(GeneratedText);
  }
}



