/** Options interface
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.top;
import xelfi.util.EnhProperties;
import java.awt.Panel;

  /** Define a class implementing this interface and register it via TopLevel.addOption(XelfiOptions).
    The top package will call it's methods when neccesary. */
public interface XelfiOptions
{
    /** This is called on xelfi start, if config file is found. Load your own values from src. Note that this
      call need not be done (if there's no config file). 
      @param src properties to load from
    */
  public void loadProperties(EnhProperties src);

    /** Save your values into properties. Called when all options are to be saved into a file. 
      @param dest properties to store into */
  public void storeProperties(EnhProperties dest);
  
    /** Return a panel where an user can edit your options or null if you don't want your 
      options to be user editable. 
      @return the panel that will be inserted into options dialog window.
    */ 
  public Panel getPanel();
  
    /** Return name of a tab with your panel or null if your getPanel() returns null. 
       @return the tab that will be inserted into options dialog window.
    */
  public String getTabName();

    /** This is called when values from the Panel p are becomming valid (use pressed OK button). 
      @param p the Panel returned by getPanel().
    */
  public void onOK(Panel p);
}

