/**
 * Xelfi
 */

package xelfi.top.toolbar;

import java.awt.*;
import java.util.Vector;
import xelfi.awt.*;
import xelfi.design.components.*;
import xelfi.design.util.*;
import xelfi.util.XelfiPath;
import xelfi.util.EnhProperties;
import xelfi.top.TopLevel;
import xelfi.top.MainMenu;

/**
 *
 * @version 	1.10, 14 Jan 1997
 * @author 	Petr Hamernik
 */
public class XFToolbar extends Panel {
    protected Color marginColor = null;
    protected static final Color defaultBorderColor = new Color(196, 196, 196);
    protected static final int indent = 8;
    protected static final int indentsep = 8;
    protected Vector data;
	protected int compW = 24, compH = 24;
    private Dimension dimLast = null;
	private boolean compSet = false;
	private int rows;

    public XFToolbar() {
        setLayout(null);
        data = new Vector();
        dimLast = new Dimension(-1,-1);
        marginColor = defaultBorderColor;
		rows = 2;
    }

	public boolean enableCommand(int index, boolean value) {
		int butCount = 0;
		for (int i = 0; i < data.size(); i++) {
			XFToolbarEntry tbe = (XFToolbarEntry)data.elementAt(i);
			if (tbe.isButton()) {
				if (tbe.getAction() == index + XFToolbarConstants.TRANS)
					((ToolbarButton)getComponent(butCount)).enableButton(value);
				butCount++;
			}
		}
		return false;
	}

	public boolean enableCommand(int index) {
		return enableCommand(index, true);
	}

	public boolean disableCommand(int index) {
		return enableCommand(index, false);
	}

    public void paint(Graphics g) {
		Dimension s = size();
		if (s != dimLast)
			setUpSizes();
		g.setColor(marginColor);
		g.draw3DRect(0,0,s.width - 1,s.height - 1, false);
		g.draw3DRect(1,1,s.width - 3,s.height - 3, true);
    }

	private Image getToolbarIcon(String fileName) {
		Image icon;
		String sep = System.getProperty("file.separator");
		String s = XelfiPath.get() + sep + "top" + sep + "toolbar" + sep + fileName + ".gif";
		icon = Toolkit.getDefaultToolkit().getImage(s);
		return icon;
	}

	public void addSeparator() {
		XFToolbarEntry tbe = new XFToolbarEntry();
		addEntry(tbe);
	}

    public void setRows(int aRows) {
		rows = aRows;
		if (rows < 1)
			rows = 1;
		setUpSizes();
		invalidate();
		getParent().validate();
	}

	public int getRows() {
		return rows;
	}

    public void addComponent(int action) {
		XFToolbarEntry tbe = new XFToolbarEntry(action);
		addEntry(tbe);
	}

        protected void addEntry(XFToolbarEntry tbe) {
          try {
		data.addElement(tbe);
		if (tbe.isButton()) {
			Image icon = getToolbarIcon(tbe.getIcon());
			if (icon == null)
				icon = getToolbarIcon("question");
			ToolbarButton b = new ToolbarButton(getToolbarIcon(tbe.getIcon()), 
				XFToolbarConstants.getHint(tbe.getAction()));
			add(b);
			Dimension d = b.preferredSize();
			if (!compSet) {
				compW = d.width;
				compH = d.height;
				compSet = true;
			}
                }
          } catch (NullPointerException ex) {
          };
	}

	public void removeAll() {
		data = new Vector();
		super.removeAll();
		compW = 24;
		compH = 24;
		compSet = false;
	}

    private void setUpSizes() {
        Dimension d = size();
        dimLast = d;
		int h = (d.height - 2 * indent) / compH;
		if (h < rows)
			h = rows;
		int x = indent;
		int y = indent;
		int pos_y = 1;
		int compcount = 0;

		for (int i = 0; i < data.size(); i++) {
			if (((XFToolbarEntry) data.elementAt(i)).isSeparator()) {
				pos_y = 1;
				y = indent;
				x += compW + indentsep;
			}
			else {
				if (pos_y > h) {
					pos_y = 1;
					y = indent;
					x += compW;
				}
				Component c = getComponent(compcount);
				compcount++;
				c.move(x, y);
				c.resize(c.preferredSize());
				pos_y++;
				y += compH;
			}
		}
    }

    public Dimension minimumSize() {
        return countSize();
    }

    public Dimension preferredSize() {
        return countSize();
    }

    private Dimension countSize() {
		int x = 2 * indent;
		int y = 2 * indent + rows * compH;
        Dimension d = size();
		int pos_y = 1;
		
		for (int i = 0; i < data.size(); i++) {
			if (((XFToolbarEntry) data.elementAt(i)).isSeparator()) {
				x += indentsep;
				pos_y = 1;
			}
			else {
				if (pos_y > rows)
					pos_y = 1;
				if (pos_y == 1)
					x += compW;
				pos_y++;
			}
		}
        return new Dimension(x,y);
    }

	public void toolbarDefault() {
		toolbarDefault(XFToolbarDefault.TOOLBAR_NORMAL);
	}

	public void toolbarDefault(int typ) {
		hide();
		removeAll();
		
		Vector tmp = XFToolbarDefault.get(typ);
		
		for (int i = 0; i < tmp.size(); i++)
			addEntry((XFToolbarEntry) tmp.elementAt(i));
		setUpSizes();
		invalidate();
		if (getParent() != null)
			getParent().validate();
		show();
	}

	public void LoadToolbar(EnhProperties prop) {
	}

	public void StoreToolbar(EnhProperties prop) {
	}

    public void resize(int width, int height) {
        super.resize(width, height);
		setUpSizes();
    }

    public void resize(Dimension d) {
        super.resize(d);
		setUpSizes();
    }

    public void reshape(int x, int y, int width, int height) {
        super.reshape(x, y, width, height);
		setUpSizes();
    }

    public boolean handleEvent(Event event) {
		if (event.id == Event.ACTION_EVENT) {
			for (int i = 0; i < countComponents(); i++)
				if ((Component) event.target == getComponent(i)) {
					int c = 0;
					for (int j = 0; j < data.size(); j++)
						if (((XFToolbarEntry) data.elementAt(j)).isButton()) {
							if (c < i)
								c++;
							else {
								int action = ((XFToolbarEntry)data.elementAt(j)).getAction()
									- XFToolbarConstants.TRANS;
								TopLevel.getMenu().itemAction(action);
								return true;
							}
						}
				}
		}
        return false;
    }

}
