/** Color Dialog Canvas
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.top.visual;

import java.awt.*;
import xelfi.awt.ColorGrid;

public class ColorDialogCanvas extends Canvas
{
  public static String sample = "Sample text";
  protected ColorGrid cg;
    // these values are used when bg or fg setting is disabled in ColorGrid and so
    // values from the ColorGrid are not valid
  private Color defaultBg, defaultFg;

  public ColorDialogCanvas(ColorGrid col)
  {
    super();
    cg = col;
  }

  public void paint(Graphics g)
  {
    FontMetrics metr = g.getFontMetrics();
    Dimension canvas = size();
    int samplew = 80, sampleh = 15, ascent = 11 ; // default values

    if(metr != null)
    {
      samplew = metr.stringWidth(sample);
      sampleh = (ascent = metr.getAscent()) + metr.getDescent();
    }

      // background
    Rectangle clip = g.getClipRect();
    g.setColor(cg.bgIsEnable() ? cg.getBgColor() : defaultBg);
    g.fillRect(clip.x, clip.y, clip.width, clip.height);
      // foreground
    g.setColor(cg.fgIsEnable() ? cg.getFgColor() : defaultFg);
    int x = (canvas.width - samplew) / 2, y = (canvas.height - sampleh) / 2 + ascent;
    g.drawString(sample, x, y);
  }
 
    /** Sets default foreground color. This color will be used when foreground setting is
      disabled in the ColorGrid. 
      @param value default foreground color.
    */
  public void setDefaultFg(Color value)
  {
    defaultFg = value;
  }

    /** Sets default background color. This color will be used when background color setting is
      disabled in the ColorGrid. 
      @param value default background color.
    */
  public void setDefaultBg(Color value)
  {
    defaultBg = value;
  }

    /** Size of the sample text */
  public Dimension minimumSize()
  {
    FontMetrics metr = getFontMetrics(getFont());
    int samplew = 80, sampleh = 15;  // default values

    if(metr != null)
    {
      samplew = metr.stringWidth(sample);
      sampleh = metr.getAscent() + metr.getDescent();
    }
    return new Dimension(samplew, sampleh);
  }

  public Dimension preferredSize()
  {
    Dimension min = minimumSize();
    min.width += 30;  // some insets
    min.height += 20;
    return min;
  }
}
