/** Color Dialog
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.top.visual;

import java.awt.List;
import java.awt.*;
import xelfi.awt.ColorGrid;
import xelfi.top.*;

public class ColorPanel extends Panel
{
  List list1;
  ColorGrid colorGrid;
  ColorDialogCanvas sample;

    // data
  Color colorValues[][];
  int tmpGridIndexes[][], gridIndexes[][], types[], spec[];

  public ColorPanel(String names[], int ttypes[], int []sspec, Color colval[][], int index[][])
  {
    colorValues = colval;
    gridIndexes = index;
    tmpGridIndexes = cloneIntArray(gridIndexes);
    types = ttypes;
    spec = sspec;

      // List
    ColLayout pl = new ColLayout();
    setLayout(pl);
    ColLayoutProperties pp = new ColLayoutProperties(new Insets(2, 2, 2, 8), 0.52f, 0.02f, 0.1f);
    list1 = new List();
    add(list1);
    pl.add(list1, pp);
      // Right panel
    Panel rp = new Panel();
    pp = new ColLayoutProperties(new Insets(0, 0, 0, 8), 0.36f);
    add(rp);
    pl.add(rp, pp);

      // Color Grid
    RowLayout rpl = new RowLayout();
    rp.setLayout(rpl);
    RowLayoutProperties rpp = new RowLayoutProperties(new Insets(2, 2, 12, 2), 0.37f);
    rpp.bottomSpaceGrowth = 0.06f;
    colorGrid = new ColorGrid();
    rp.add(colorGrid);
    rpl.add(colorGrid, rpp);
      // Text sample
    sample = new ColorDialogCanvas(colorGrid);
    rpp = new RowLayoutProperties(new Insets(6, 2, 2, 2), 0.57f, 0, 0.03f);
    rp.add(sample);
    rpl.add(sample, rpp);

    int i;
    for(i = 0; i < names.length; i++) list1.addItem(names[i]);
  }

  public void show()
  {
    super.show();
  }

  public Dimension preferredSize()
  {
    Dimension size = super.preferredSize();
    if(size.width < 420) size.width = 420;
    if(size.height < 302) size.height = 302;
    return size;
  }

  public void saveValues()
  {
    int i;
    for(i = 0; i < colorValues.length; i++)
    {
      if((types[i] & XFColor.TYPE_BG) != 0)
      {
        colorValues[i][XFColor.BG] = colorGrid.getGridColor(tmpGridIndexes[i][XFColor.BG]);
        gridIndexes[i][XFColor.BG] = tmpGridIndexes[i][XFColor.BG];
      }
      if((types[i] & XFColor.TYPE_FG) != 0)
      {
        colorValues[i][XFColor.FG] = colorGrid.getGridColor(tmpGridIndexes[i][XFColor.FG]);
        gridIndexes[i][XFColor.FG] = tmpGridIndexes[i][XFColor.FG];
      }
    }
  }

  public boolean handleEvent(Event evt)
  {
    switch(evt.id)
    {
      case ColorGrid.COL_GRID_CHANGE:
        colChange();
        return true;
      case Event.LIST_SELECT:
        if(evt.target == list1)
        {
          listSelection(((Integer)evt.arg).intValue());
          return true;
        }
    }
    return super.handleEvent(evt);
  }

  protected void colChange()
  {
    sample.repaint();

    try
    {
      int i = list1.getSelectedIndex();
      tmpGridIndexes[i][XFColor.BG] = colorGrid.getBg();
      tmpGridIndexes[i][XFColor.FG] = colorGrid.getFg();
    }
    catch(ArrayIndexOutOfBoundsException e) { }
  }

  protected void listSelection(int i)
  {
    if(!((types[i] & XFColor.TYPE_FG) != 0))  // background only
    {
      colorGrid.setBg(tmpGridIndexes[i][XFColor.BG]);
      colorGrid.setFg(tmpGridIndexes[spec[i]][XFColor.FG]);  // use another's color foreground
      sample.setDefaultFg(colorGrid.getGridColor(tmpGridIndexes[spec[i]][XFColor.FG]));
    }

    if(!((types[i] & XFColor.TYPE_BG) != 0))  // foreground only
    {
      colorGrid.setFg(tmpGridIndexes[i][XFColor.FG]);
      colorGrid.setBg(tmpGridIndexes[spec[i]][XFColor.BG]);  // use another's color foreground
      sample.setDefaultBg(colorGrid.getGridColor(tmpGridIndexes[spec[i]][XFColor.BG]));
    }

    if((types[i] & XFColor.TYPE_BOTH) == XFColor.TYPE_BOTH)  // both
    {
      colorGrid.setFg(tmpGridIndexes[i][XFColor.FG]);
      colorGrid.setBg(tmpGridIndexes[i][XFColor.BG]);  // use another's color foreground
    }
    colorGrid.fgEnable((types[i] & XFColor.TYPE_FG) != 0);
    colorGrid.bgEnable((types[i] & XFColor.TYPE_BG) != 0);
    
    sample.repaint();
  }

  protected int[][] cloneIntArray(int src[][])
  {
    int clon[][] = new int[src.length][];
    int i, j;
    for(i = 0; i < src.length; i++)
    {
      clon[i] = new int[src[i].length];
      for(j = 0; j < src[i].length; j++) clon[i][j] = src[i][j];
    }
    return clon;
  }
}
