/** New package Dialog
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.top.visual;

import java.awt.*;
import java.io.*;

import xelfi.editor.XelfiFile;
import xelfi.util.MessageBox;
import xelfi.browser.document.*;
import xelfi.browser.view.PackageView;

public class NewPackageDialog extends Dialog
{
  List packages;
  TextField packname;
  Button butOK, butCanc;

  private boolean useRootPackage;
  private BrPackage selected[];

  public NewPackageDialog(Frame par)
  {
    super(par, "New package", true);

    packages = new List();
    selected = ComponentManager.getSelectedPackages();
    if(selected == null || selected.length == 0)
    {
      packages.addItem("<Root>");
      useRootPackage = true;
      // root package
    }
    else
    {
      for(int i = 0; i < selected.length; i++)
      {
        String tmp = selected[i].getXFile().getPackage();
        if(tmp.length() == 0) packages.addItem("<Root>");
        else packages.addItem(tmp);
      }
      useRootPackage = false;
    }

    RowLayout dl = new RowLayout();
    setLayout(dl);
    Insets di1 = new Insets(2, 2, 2, 6);
    Insets di2 = new Insets(0, 8, 4, 6);
    RowLayoutProperties dp = new RowLayoutProperties(di1, 0, 0.02f, 0);
    dp.leftSpaceGrowth = dp.rightSpaceGrowth = 0.03f;

      // package list
    Label tf = new Label("Create in");
    add(tf);
    dl.add(tf, dp);
    dp.compInsets = di2;
    dp.topSpaceGrowth = 0;
    dp.compGrowth = 0.81f;
    add(packages);
    dl.add(packages, dp);
    dp.compGrowth = 0;

      // class name
    tf = new Label("Package name");
    dp.topSpaceGrowth = 0.04f;
    dp.compInsets = di1;
    add(tf);
    dl.add(tf, dp);
    dp.compInsets = di2;
    dp.topSpaceGrowth = 0;
    packname = new TextField();
    add(packname);
    dl.add(packname, dp);

      // button panel
    Panel bp = new Panel();
    dp.topSpaceGrowth = 0.06f;
    dp.bottomSpaceGrowth = 0.05f;
    dp.compGrowth = 0.02f;
    dp.compInsets = new Insets(6, 2, 3, 2);
    add(bp);
    dl.add(bp, dp);

      // button panel setup
    ColLayout bpl = new ColLayout();
    ColLayoutProperties bpp = new ColLayoutProperties(new Insets(0, 2, 0, 2), 0.02f);
    bpp.leftSpaceGrowth = 0.41f;
    bpp.rightSpaceGrowth = 0.03f;
    butOK = new Button(" OK ");
    bp.add(butOK);
    bpl.add(butOK, bpp);
    bpp.compGrowth = 0.03f;
    bpp.leftSpaceGrowth = 0;
    bpp.rightSpaceGrowth = 0.41f;
    butCanc = new Button("Cancel");
    bp.add(butCanc);
    bpl.add(butCanc, bpp);

    packages.select(0);
    pack();
  }

  public boolean action(Event e, Object arg)
  {
    if(e.target == butOK)
    {
      newPackage();
      return true;
    }
    else if(e.target == butCanc)
    {
      hide();
      dispose();
      return true;
    }
    return super.action(e, arg);
  }

  public void show()
  {
    if(getPeer() == null) addNotify();
    Dimension scr = Toolkit.getDefaultToolkit().getScreenSize(), me = size();

    move((scr.width - me.width) / 2, (int)((scr.height - me.height) / 2.4));
    super.show();
    packname.requestFocus();
  }

  public boolean keyDown(Event  e, int  key)
  {
    if(key == '\n') 
    {
      handleEvent(new Event(butOK, Event.ACTION_EVENT, null));
      return true;
    }
    else return false;
  }

  public boolean handleEvent(Event evt)
  {
    switch(evt.id)
    {
      case Event.WINDOW_DESTROY:
        if(evt.target == this)
        {
          hide();
          dispose();
          return true;
        }
    }
    return super.handleEvent(evt);
  }

  protected void newPackage()
  {
    String name = packname.getText();
    int parentPack = packages.getSelectedIndex();

    if(name.equals("") || parentPack == -1) return;

    if(useRootPackage)  // my own way to create packages
    {
      String rootPath = XFile.getPreferredPath();
      File newPackage = new File(rootPath, name);
      if(!newPackage.mkdir()) new MessageBox("Error", "Can't create package " +
        name + " (at " + newPackage.toString() + ")").show();
    }
    else  // package will be created by browser
      PackageView.createPackage(selected[parentPack], name);

    hide();
    dispose();
  }

  public Dimension preferredSize()
  {
    Dimension pref = super.preferredSize();
    if(pref.width < 260) pref.width = 260;
    return pref;
  }

  public Insets insets()   // Under JDK 1.0.2 for win95 insets doesnt work. This patch should help.
  {
    Insets i = super.insets();
    if(getPeer() != null && (i.top < 0 || i.bottom < 0 || i.left < 0 || i.right < 0))
      return new Insets(28, 4, 4, 4);
    else return i;
  }
}


